package com.hz.pm.api.common.statemachine.action;

import cn.hutool.core.lang.Assert;
import com.hz.pm.api.common.statemachine.builder.impl.TenderStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.ProjectStatusChangeEvent;
import com.hz.pm.api.common.statemachine.util.TenderStateMachineUtil;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.TenderStatusEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.OnTransition;
import org.springframework.statemachine.annotation.WithStateMachine;

/**
 * <p>
 * 标段状态机action集合类
 * </p>
 *
 * @author WendyYang
 * @since 17:08 2024/2/24
 */
@Slf4j
@WithStateMachine(id = TenderStateMachineBuilderImpl.MACHINE_ID)
public class TenderAction {

    private Purchase getPurchaseInfo(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = (Purchase) message.getHeaders().get(TenderStateMachineUtil.PURCHASE);
        Assert.notNull(purchase, "未获取到需要状态变更的标段信息");
        return purchase;
    }

    /**
     * 填写采购信息
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_PURCHASE_INFO", target = "TO_BE_SUBMIT_CONSTRUCTION_INFO")
    public void SUBMIT_PURCHASE_INFO(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.TO_BE_SUBMIT_CONSTRUCTION_INFO.getTenderStatus());
    }

    /**
     * 填写合同信息
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_CONSTRUCTION_INFO", target = "TO_BE_SUBMIT_OPERATION_PLAN")
    public void SUBMIT_CONSTRUCTION_INFO(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.TO_BE_SUBMIT_OPERATION_PLAN.getTenderStatus());
    }

    /**
     * 填写实施计划
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_OPERATION_PLAN", target = "TO_BE_SUBMIT_FIRST_INSPECTED_INFO")
    public void SUBMIT_OPERATION_PLAN(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.TO_BE_SUBMIT_FIRST_INSPECTED_INFO.getTenderStatus());
    }

    /**
     * 上传初验材料
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_FIRST_INSPECTED_INFO", target = "TO_BE_SUBMIT_XCFHX_APPLY")
    public void SUBMIT_FIRST_INSPECTED_FILES(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.TO_BE_SUBMIT_XCFHX_APPLY.getTenderStatus());
    }

    /**
     * 信创符合性申请
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_XCFHX_APPLY", target = "ON_XCFHX_APPLY")
    public void SUBMIT_XCFHX_APPLY(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.ON_XCFHX_APPLY.getTenderStatus());
    }

    /**
     * 信创符合性申请通过
     *
     * @param message \
     */
    @OnTransition(source = "ON_XCFHX_APPLY", target = "TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY")
    public void XCFHX_APPLY_PASSED(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY.getTenderStatus());
    }

    /**
     * 信创符合性申请失败
     *
     * @param message \
     */
    @OnTransition(source = "ON_XCFHX_APPLY", target = "XCFHX_APPLY_FAILED")
    public void XCFHX_APPLY_FAILED(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.XCFHX_APPLY_FAILED.getTenderStatus());
    }

    /**
     * 重新提交信创符合性申请
     *
     * @param message \
     */
    @OnTransition(source = "XCFHX_APPLY_FAILED", target = "ON_XCFHX_APPLY")
    public void RESUBMIT_XCFHX_APPLY(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.ON_XCFHX_APPLY.getTenderStatus());
    }

    /**
     * 终验申报
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY", target = "ON_FINALLY_INSPECTED_APPLY")
    public void SUBMIT_FINALLY_INSPECTED(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.ON_FINALLY_INSPECTED_APPLY.getTenderStatus());
    }

    /**
     * 终验申报重新提交
     *
     * @param message \
     */
    @OnTransition(source = "FINALLY_INSPECTED_FAILED", target = "ON_FINALLY_INSPECTED_APPLY")
    public void RESUBMIT_FINALLY_INSPECTED(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.ON_FINALLY_INSPECTED_APPLY.getTenderStatus());
    }

    /**
     * 终验申报通过
     *
     * @param message \
     */
    @OnTransition(source = "ON_FINALLY_INSPECTED_APPLY", target = "FINALLY_INSPECTED_PASSED")
    public void FINALLY_INSPECTED_PASSED(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.FINALLY_INSPECTED_PASSED.getTenderStatus());
    }

    /**
     * 终验申报拒绝
     *
     * @param message \
     */
    @OnTransition(source = "ON_FINALLY_INSPECTED_APPLY", target = "FINALLY_INSPECTED_FAILED")
    public void FINALLY_INSPECTED_FAILED(Message<ProjectStatusChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatusEnum.FINALLY_INSPECTED_FAILED.getTenderStatus());
    }

}
