package com.hz.pm.api.common.statemachine.event;

import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Objects;

/**
 * <p>
 * TenderStatusChangeEvent
 * </p>
 *
 * @author WendyYang
 * @since 00:14 2024/2/24
 */
@Getter
@AllArgsConstructor
public enum TenderStatusChangeEvent {

    /**
     * 填写采购信息
     */
    SUBMIT_PURCHASE_INFO(101, null, null),
    /**
     * 填写合同信息
     */
    SUBMIT_CONSTRUCTION_INFO(102, null, null),
    /**
     * 填写实施计划
     */
    SUBMIT_OPERATION_PLAN(103, null, null),
    /**
     * 上传初验材料
     */
    SUBMIT_FIRST_INSPECTED_FILES(201, null, null),
    /**
     * 信创符合性申请
     */
    SUBMIT_XCFHX_APPLY(301, null, null),
    /**
     * 信创符合性重新申请
     */
    RESUBMIT_XCFHX_APPLY(303, null, null),
    /**
     * 信创符合性申请通过
     */
    XCFHX_APPLY_PASSED(302, null, null),
    /**
     * 信创符合性申请不通过
     */
    XCFHX_APPLY_FAILED(null, 302, null),
    /**
     * 终验申报
     */
    SUBMIT_FINALLY_INSPECTED(401, null, null),
    /**
     * 终验申报重新提交
     */
    RESUBMIT_FINALLY_INSPECTED(502, null, null),
    /**
     * 终验申报通过
     */
    FINALLY_INSPECTED_PASSED(501, null, null),
    /**
     * 终验申报拒绝
     */
    FINALLY_INSPECTED_FAILED(null, 501, null);


    private final Integer passStatusCode;
    private final Integer rejectStatusCode;
    private final Integer withdrawStatusCode;

    public static TenderStatusChangeEvent getPassValueByCode(Integer code) {
        if (Objects.isNull(code)) {
            return null;
        }
        for (TenderStatusChangeEvent t : TenderStatusChangeEvent.values()) {
            if (code.equals(t.getPassStatusCode())) {
                return t;
            }
        }
        return null;
    }

    public static TenderStatusChangeEvent getRejectValueByCode(Integer code) {
        if (Objects.isNull(code)) {
            return null;
        }
        for (TenderStatusChangeEvent t : TenderStatusChangeEvent.values()) {
            if (code.equals(t.getRejectStatusCode())) {
                return t;
            }
        }
        return null;
    }

    public static TenderStatusChangeEvent getWithdrawValueByCode(Integer code) {
        if (Objects.isNull(code)) {
            return null;
        }
        for (TenderStatusChangeEvent t : TenderStatusChangeEvent.values()) {
            if (code.equals(t.getWithdrawStatusCode())) {
                return t;
            }
        }
        return null;
    }

}
