package com.hz.pm.api.external;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.hz.pm.api.external.model.dto.*;
import lombok.extern.slf4j.Slf4j;
import org.omg.CORBA.TIMEOUT;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;

import static cn.hutool.core.date.DatePattern.NORM_DATETIME_FORMATTER;

/**
 * <p>
 * MhApiClient
 * </p>
 *
 * @author WendyYang
 * @since 20:41 2023/12/20
 */
@Slf4j
@Component
public class MhApiClient {

    @Value("${mh.api-host:}")
    private String mhApiHost;

    @Value("${mh.zwdd-work-notice.client-id:}")
    private String zwddWorkNoticeClientId;

    @Value("${mh.zwdd-work-notice.client-secret:}")
    private String zwddWorkNoticeClientSecret;

    private static final String UNIT_URL = "/sync/unit";
    private static final String USER_URL = "/sync/user";
    private static final String EXPERT_URL = "/sync/expert";
    private static final String COMPANY_URL = "/sync/company";
    private static final String PROJECT_URL = "/project_info/getProjectInfoList";

    private static final String ZWDD_MSG_URL = "/message/send";


    public MhRetDTO<List<MhUnitDTO>> queryUnits() {
        String requestUrl = mhApiHost + UNIT_URL;
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhUnitDTO>>>() {
        }, false);
    }

    public MhRetDTO<List<MhCompanyDTO>> queryCompany() {
        String requestUrl = mhApiHost + COMPANY_URL;
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhCompanyDTO>>>() {
        }, false);
    }

    public MhRetDTO<List<MhUserDTO>> queryUsers(LocalDateTime syncDateTime) {
        String requestUrl = mhApiHost + USER_URL;
        if (syncDateTime != null) {
            String timeString = LocalDateTimeUtil.format(syncDateTime, NORM_DATETIME_FORMATTER);
            requestUrl += "?syncDateTime=" + timeString;
        }
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhUserDTO>>>() {
        }, false);
    }

    public MhRetDTO<MhExpertDTO> queryExperts(LocalDateTime syncDateTime) {
        String requestUrl = mhApiHost + EXPERT_URL;
        if (syncDateTime != null) {
            String timeString = LocalDateTimeUtil.format(syncDateTime, NORM_DATETIME_FORMATTER);
            requestUrl += "?syncDateTime=" + timeString;
        }
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<MhExpertDTO>>() {
        }, false);
    }

    public MhRetDTO<List<MhProjectDTO>> listProjects() {
        String requestUrl = mhApiHost + PROJECT_URL;
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhProjectDTO>>>() {
        }, false);
    }

    public boolean sendZwddWorkNotice(MhZwddWorkNoticeDTO notice) {
        notice.setClientId(zwddWorkNoticeClientId);
        notice.setClientSecret(zwddWorkNoticeClientSecret);
        String retBody = HttpUtil.post(mhApiHost + ZWDD_MSG_URL, JSONUtil.toJsonStr(notice));
        JSONObject retObj = JSONUtil.parseObj(retBody);
        if (retObj.containsKey("code") && retObj.getInt("code") == 200) {
            return Boolean.TRUE;
        }
        log.warn("发送浙政钉工作通知失败：{}", retObj);
        return Boolean.FALSE;
    }


}
