package com.hz.pm.api.projectlib.handle;

import com.hz.pm.api.common.model.constant.CommonConst;
import com.hz.pm.api.common.statemachine.event.ProjectStatusChangeEvent;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.ProjectStatusEnum;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.enums.StepStatusEnum;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * 立项批复处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Order(8)
@Component
public class ProjectApprovalHandle extends AbstractProcessBusinessHandle {
    private final IProjectStatusChangeService projectStatusChangeService;
    private final IProjectService projectService;

    private Integer order = 8;

    public ProjectApprovalHandle(IProjectStatusChangeService projectStatusChangeService,
                                 IProjectService projectService) {
        this.projectStatusChangeService = projectStatusChangeService;
        this.projectService = projectService;
    }

    @Override
    void businessHandle(Project project, List<ProcessDetailVO> processSchedule) {
        ProcessDetailVO processDetail = new ProcessDetailVO();
        processDetail.setProcessName(CommonConst.PROJECT_APPROVAL);
        processSchedule.add(processDetail);
        //查出历史版本的 所有项目ID
        List<Long> allProjectIds = projectService.allVersionProjectIds(project.getProjectCode());
        Integer status = project.getStatus();
        if (Objects.isNull(status)) {
            return;
        }
        // 项目状态为待立项批复之前的状态
        if (status < ProjectStatusEnum.TO_BE_APPROVED.getCode()
                && status < ProjectStatusEnum.DECLARED_APPROVED_TO_BE_RECORD.getCode()) {
            processDetail.setStepStatus(StepStatusEnum.NOT_START);
            return;
        }
        // 当前项目状态为待立项批复
        if (ProjectStatusEnum.TO_BE_APPROVED.eq(status)
                || ProjectStatusEnum.DECLARED_APPROVED_TO_BE_RECORD.eq(status)
                || ProjectStatusEnum.DECLARED_APPROVED_RECORD_AUDITING.eq(status)) {
            // 根据建设方案评审通过的时间获取
            final List<ProjectStatusChangeEvent> events = Arrays.asList(ProjectStatusChangeEvent.PLAN_REVIEW_PASS,
                    ProjectStatusChangeEvent.DECLARED_RECORD_SUBMIT,
                    ProjectStatusChangeEvent.DECLARED_RECORD_RESUBMIT);
            ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(allProjectIds, events);
            if (projectStatusChange != null) {
                processDetail.setFinishTime(projectStatusChange.getCreateOn());
            }
            processDetail.setStepStatus(StepStatusEnum.ON_GOING);
        } else {
            // 取 状态机 改变状态时间
            final List<ProjectStatusChangeEvent> events = Arrays.asList(ProjectStatusChangeEvent.PLAN_REVIEW_PASS,
                    ProjectStatusChangeEvent.DECLARED_RECORD_FAILED,
                    ProjectStatusChangeEvent.DECLARED_RECORD_PASS);
            ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(allProjectIds, events);
            if (Objects.nonNull(projectStatusChange)) {
                processDetail.setFinishTime(projectStatusChange.getCreateOn());
                processDetail.setStepStatus(StepStatusEnum.COMPLETED);
            }
        }
    }

    @Override
    Integer getOrder() {
        return order;
    }

    @Override
    void setOrder(Integer order) {
        this.order = order;
    }
}
