package com.hz.pm.api.projectlib.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.projectlib.model.entity.MhProject;

import java.util.Optional;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author WendyYang
 * @since 2024-01-30
 */
public interface IMhProjectService extends IService<MhProject> {

    default MhProject getByProjectCode(String projectCode) {
        LambdaQueryWrapper<MhProject> query = Wrappers.lambdaQuery(MhProject.class)
                .eq(MhProject::getProjectCode, projectCode);
        return getOne(query);
    }

    default Optional<Long> getIdByProjectCode(String projectCode) {
        LambdaQueryWrapper<MhProject> query = Wrappers.lambdaQuery(MhProject.class)
                .select(MhProject::getId)
                .eq(MhProject::getProjectCode, projectCode);
        return Optional.ofNullable(getOne(query))
                .flatMap(w -> Optional.of(w.getId()));
    }

    default boolean existsByProjectCode(String projectCode) {
        LambdaQueryWrapper<MhProject> query = Wrappers.lambdaQuery(MhProject.class)
                .eq(MhProject::getProjectCode, projectCode)
                .last(BizConst.LIMIT_1);
        return count(query) > 0;
    }

}
